import { JSX, Accessor } from 'solid-js';

type TokenComponent<T extends {}> = (() => JSX.Element) & T;
/**
 * Provides the tools to create tokenized components, parse and identify tokens in JSX Elements.
 *
 * @param options - Additional options for the parser
 * @param options.name - The name of the parser, used for debugging
 * @returns functions `createToken`, `childrenTokens`, `isToken` and the symbol associated with the jsx-parser `id`.
 * @example
 * ```tsx
 * const { createToken, childrenTokens } = createJSXParser();
 *
 * const MyToken = createToken(props => ({ type: "my-token", props }));
 *
 * const MyComponent = (props) => {
 *  const tokens = childrenTokens(() => props.children);
 *  return <ul>{tokens().map(token => <li>token.type</li>)}</ul>;
 * }
 *
 * <MyComponent>
 *   <MyToken/>
 *   <MyToken/>
 * </MyComponent>
 * ```
 */
declare function createJSXParser<Tokens extends {}>(options?: {
    name: string;
}): {
    createToken: <Props extends {
        [key: string]: any;
    }, Token extends Tokens>(tokenCallback: (props: Props) => Token, component?: ((props: Props) => JSX.Element) | undefined) => (props: Props) => JSX.Element;
    childrenTokens: (fn: Accessor<JSX.Element>) => Accessor<TokenComponent<Tokens>[]>;
    isToken: (value: unknown | TokenComponent<Tokens>) => value is TokenComponent<Tokens>;
    id: symbol;
};

export { TokenComponent, createJSXParser };
